@@ -1,6 +1,6 @@
 ;;; helm-find.el --- helm interface for find command. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2018 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2019 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -31,6 +31,12 @@ I.e use the -path/ipath arguments of find instead of -name/iname."
   :group 'helm-files
   :type 'boolean)
 
+(defvar helm-find-map
+  (let ((map (make-sparse-keymap)))
+    (set-keymap-parent map helm-generic-files-map)
+    (define-key map (kbd "DEL") 'helm-delete-backward-no-update)
+    map))
+
 (defvar helm-source-findutils
   (helm-build-async-source "Find"
     :header-name (lambda (name)
@@ -40,7 +46,8 @@ I.e use the -path/ipath arguments of find instead of -name/iname."
     :action-transformer 'helm-transform-file-load-el
     :persistent-action 'helm-ff-kill-or-find-buffer-fname
     :action 'helm-type-file-actions
-    :keymap helm-generic-files-map
+    :help-message 'helm-generic-file-help-message
+    :keymap helm-find-map
     :candidate-number-limit 9999
     :requires-pattern 3))
 
