@@ -1,6 +1,6 @@
 ;;; helm-locate.el --- helm interface for locate. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2018 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2019 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -127,37 +127,11 @@ help for more infos."
   :group 'helm-files)
 
 
-(defvar helm-generic-files-map
+(defvar helm-locate-map
   (let ((map (make-sparse-keymap)))
-    (set-keymap-parent map helm-map)
-    (define-key map (kbd "C-]")     'helm-ff-run-toggle-basename)
-    (define-key map (kbd "C-s")     'helm-ff-run-grep)
-    (define-key map (kbd "M-g s")   'helm-ff-run-grep)
-    (define-key map (kbd "M-g z")   'helm-ff-run-zgrep)
-    (define-key map (kbd "M-g p")   'helm-ff-run-pdfgrep)
-    (define-key map (kbd "C-c g")   'helm-ff-run-gid)
-    (define-key map (kbd "M-R")     'helm-ff-run-rename-file)
-    (define-key map (kbd "M-C")     'helm-ff-run-copy-file)
-    (define-key map (kbd "M-B")     'helm-ff-run-byte-compile-file)
-    (define-key map (kbd "M-L")     'helm-ff-run-load-file)
-    (define-key map (kbd "M-S")     'helm-ff-run-symlink-file)
-    (define-key map (kbd "M-H")     'helm-ff-run-hardlink-file)
-    (define-key map (kbd "M-D")     'helm-ff-run-delete-file)
-    (define-key map (kbd "C-=")     'helm-ff-run-ediff-file)
-    (define-key map (kbd "C-c =")   'helm-ff-run-ediff-merge-file)
-    (define-key map (kbd "C-c o")   'helm-ff-run-switch-other-window)
-    (define-key map (kbd "C-c r")   'helm-ff-run-find-file-as-root)
-    (define-key map (kbd "C-c C-o") 'helm-ff-run-switch-other-frame)
-    (define-key map (kbd "M-i")     'helm-ff-properties-persistent)
-    (define-key map (kbd "C-c C-x") 'helm-ff-run-open-file-externally)
-    (define-key map (kbd "C-c X")   'helm-ff-run-open-file-with-default-tool)
-    (define-key map (kbd "M-.")     'helm-ff-run-etags)
-    (define-key map (kbd "C-c @")   'helm-ff-run-insert-org-link)
-    (define-key map (kbd "C-x C-q") 'helm-ff-run-marked-files-in-dired)
-    (define-key map (kbd "C-c C-a") 'helm-ff-run-mail-attach-files)
-    map)
-  "Generic Keymap for files.")
-
+    (set-keymap-parent map helm-generic-files-map)
+    (define-key map (kbd "DEL") 'helm-delete-backward-no-update)
+    map))
 
 (defface helm-locate-finish
     '((t (:foreground "Green")))
@@ -355,7 +329,9 @@ See also `helm-locate'."
 Sort is done on basename of CANDIDATES."
   (helm-fuzzy-matching-default-sort-fn-1 candidates nil t))
 
-(defclass helm-locate-source (helm-source-async helm-type-file)
+(defclass helm-locate-override-inheritor (helm-type-file) ())
+
+(defclass helm-locate-source (helm-source-async helm-locate-override-inheritor)
   ((init :initform 'helm-locate-initial-setup)
    (candidates-process :initform 'helm-locate-init)
    (requires-pattern :initform 3)
@@ -365,6 +341,10 @@ Sort is done on basename of CANDIDATES."
    (redisplay :initform (progn helm-locate-fuzzy-sort-fn))
    (group :initform 'helm-locate)))
 
+;; Override helm-type-file class keymap.
+(defmethod helm--setup-source :after ((source helm-locate-override-inheritor))
+  (setf (slot-value source 'keymap) helm-locate-map))
+
 (defvar helm-source-locate
   (helm-make-source "Locate" 'helm-locate-source
     :pattern-transformer 'helm-locate-pattern-transformer
