@@ -1,6 +1,6 @@
 ;;; helm-mode.el --- Enable helm completion everywhere. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2018 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2019 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -23,7 +23,8 @@
 (require 'helm-files)
 
 (defvar crm-separator)
-
+(defvar ido-everywhere)
+(declare-function ido-mode "ido.el")
 
 (defgroup helm-mode nil
   "Enable helm completion."
@@ -52,7 +53,10 @@
     (dired-do-copy . helm-read-file-name-handler-1)
     (dired-do-symlink . helm-read-file-name-handler-1)
     (dired-do-relsymlink . helm-read-file-name-handler-1)
-    (dired-do-hardlink . helm-read-file-name-handler-1))
+    (dired-do-hardlink . helm-read-file-name-handler-1)
+    (basic-save-buffer . helm-read-file-name-handler-1)
+    (write-file . helm-read-file-name-handler-1)
+    (write-region . helm-read-file-name-handler-1))
   "Completing read functions for specific Emacs commands.
 
 By default `helm-mode' use `helm-completing-read-default-handler' to
@@ -60,7 +64,7 @@ provide helm completion in each `completing-read' or `read-file-name'
 found, but other functions can be specified here for specific
 commands. This also allow disabling helm completion for some commands
 when needed.
- 
+
 Each entry is a cons cell like (EMACS_COMMAND . COMPLETING-READ_HANDLER)
 where key and value are symbols.
 
@@ -102,7 +106,7 @@ Example:
 
 We want here to make the regular `completing-read' in `foo/test'
 returns a list of candidate(s) instead of a single candidate.
- 
+
 Note that this function will be reused for ALL the `completing-read'
 of this command, so it should handle all cases, e.g
 If first `completing-read' complete against symbols and
@@ -561,12 +565,12 @@ that use `helm-comp-read' See `helm-M-x' for example."
            (get-candidates
             (lambda ()
               (let ((cands (helm-comp-read-get-candidates
-                            collection test sort alistp
-                            ;; This should not be needed as
-                            ;; `helm-pattern' is not yet computed when
-                            ;; calling this from :init when
-                            ;; candidates-in-buffer is in use.
-                            (if candidates-in-buffer "" helm-pattern))))
+                            ;; If `helm-pattern' is passed as INPUT
+                            ;; and :alistp is nil INPUT is passed to
+                            ;; `all-completions' which defeat helm
+                            ;; matching functions (multi match, fuzzy
+                            ;; etc...) issue #2134.
+                            collection test sort alistp "")))
                 (helm-cr-default default cands))))
            (history-get-candidates
             (lambda ()
@@ -899,7 +903,7 @@ See documentation of `completing-read' and `all-completions' for details."
                 def-com
                 (apply def-com def-args))
                (;; Use by default a in-buffer handler unless
-                ;; COLLECTION is a function. 
+                ;; COLLECTION is a function.
                 t
                 (funcall default-handler
                          prompt collection predicate require-match
@@ -1132,7 +1136,6 @@ Don't use it directly, use instead `helm-read-file-name' in your programs."
          ;; Append the two extra args needed to set the buffer and source name
          ;; in helm specialized functions.
          (any-args (append def-args (list str-command buf-name)))
-         (ido-state ido-mode)
          helm-completion-mode-start-message ; Be quiet
          helm-completion-mode-quit-message  ; Same here
          fname)
@@ -1148,7 +1151,7 @@ Don't use it directly, use instead `helm-read-file-name' in your programs."
     ;; we switch to `helm-read-file-name' and don't try to call it
     ;; with wrong number of args.
     (when (eq def-com 'ido)
-      (setq def-com 'ido-read-file-name) (ido-mode 1))
+      (setq def-com 'ido-read-file-name))
     (when (and def-com (> (length (help-function-arglist def-com)) 8))
       (setq def-com 'incompatible))
     (unless (or (not entry) def-com)
@@ -1197,8 +1200,8 @@ Don't use it directly, use instead `helm-read-file-name' in your programs."
                        :alistp nil
                        :must-match mustmatch
                        :test predicate))))
+      (and ido-mode (ido-mode -1))
       (helm-mode 1)
-      (ido-mode (if ido-state 1 -1))
       ;; Same comment as in `helm--completing-read-default'.
       (setq this-command current-command))
     (if (and
@@ -1382,6 +1385,21 @@ Can be used as value for `completion-in-region-function'."
 (when (boundp 'completion-in-region-function)
   (defconst helm--old-completion-in-region-function completion-in-region-function))
 
+(defun helm-mode--disable-ido-maybe (&optional from-hook)
+  (when (and (boundp 'ido-everywhere) ido-everywhere)
+    (remove-function read-file-name-function #'ido-read-file-name)
+    (remove-function read-buffer-function #'ido-read-buffer)
+    (setq ido-everywhere nil)
+    (if from-hook
+        (user-error "Unable to turn on Ido-everywhere while Helm-mode is enabled")
+      (user-error "Helm-mode enabled (Ido-everywhere is incompatible with Helm-mode, disabling it)"))))
+
+(defun helm-mode--ido-everywhere-hook ()
+  ;; Called only when user calls directly ido-everywhere
+  ;; and helm-mode is enabled.
+  (when helm-mode
+    (helm-mode--disable-ido-maybe t)))
+
 ;;;###autoload
 (define-minor-mode helm-mode
     "Toggle generic helm completion.
@@ -1428,7 +1446,15 @@ Note: This mode is incompatible with Emacs23."
                           #'helm--generic-read-buffer)
             (when helm-mode-handle-completion-in-region
               (add-function :override completion-in-region-function
-                            #'helm--completion-in-region)))
+                            #'helm--completion-in-region))
+            ;; If user have enabled ido-everywhere BEFORE enabling
+            ;; helm-mode disable it and warn user about its
+            ;; incompatibility with helm-mode (issue #2085).
+            (helm-mode--disable-ido-maybe)
+            ;; If ido-everywhere is not enabled yet anticipate and
+            ;; disable it if user attempt to enable it while helm-mode
+            ;; is running (issue #2085).
+            (add-hook 'ido-everywhere-hook #'helm-mode--ido-everywhere-hook))
           (setq completing-read-function 'helm--completing-read-default
                 read-file-name-function  'helm--generic-read-file-name
                 read-buffer-function     'helm--generic-read-buffer)
@@ -1441,7 +1467,8 @@ Note: This mode is incompatible with Emacs23."
             (remove-function completing-read-function #'helm--completing-read-default)
             (remove-function read-file-name-function #'helm--generic-read-file-name)
             (remove-function read-buffer-function #'helm--generic-read-buffer)
-            (remove-function completion-in-region-function #'helm--completion-in-region))
+            (remove-function completion-in-region-function #'helm--completion-in-region)
+            (remove-hook 'ido-everywhere-hook #'helm-mode--ido-everywhere-hook))
           (setq completing-read-function (and (fboundp 'completing-read-default)
                                         'completing-read-default)
                 read-file-name-function  (and (fboundp 'read-file-name-default)
